import java.awt.*;
import java.awt.geom.*;

/**
 * A rectangle that can be manipulated and drawn.
 * @author Joel Ross
 * @author adapted from Michael Kolling and David J. Barnes
 * @version Sp 2015
 *
 */
public class Rectangle extends DrawableShape
{
    //instance variables
    private int xPosition;
    private int yPosition;
    private int width;
    private int height;
    private String color;
    private boolean filled;

    /**
     * Creates a new Rectangle object with default location, radius, and color.
     */
    public Rectangle()
    {
        //give instance variables default values
        this.xPosition = 60;
        this.yPosition = 50;
        this.width = 50;
        this.height = 30;
        this.color = "red";
        this.filled = true;

        this.addSelfToCanvas();
    }

    /**
     * Returns the rectangle's current x position
     */
    public int getXPosition()
    {
        return xPosition;
    }

    /**
     * Sets the rectangle's x position to the given value
     */
    public void setXPosition(int newX)
    {
        this.xPosition = newX;
        this.redraw();
    }

    /**
     * Returns the rectangle's current y position
     */
    public int getYPosition()
    {
        return yPosition;
    }

    /**
     * Sets the rectangle's y position to the given value
     */
    public void setYPosition(int newY)
    {
        this.yPosition = newY;
        this.redraw();
    }

    /**
     * Returns the rectangle's current width
     */
    public int getWidth()
    {
        return width;
    }

    /**
     * Sets the rectangle's width to the given value
     */
    public void setWidth(int newWidth)
    {
        this.width = newWidth;
        this.redraw();
    }

    /**
     * Returns the rectangle's current height
     */
    public int getHeight()
    {
        return height;
    }

    /**
     * Sets the rectangle's height to the given value
     */
    public void setHeight(int newHeight)
    {
        this.height = newHeight;
        this.redraw();
    }

    /**
     * Returns the rectangle's current color (as a String). Can either be: 
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public String getColorName()
    {
        return color;
    }

    /**
     * Set the circle's color to the given value. Can either be:
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public void setColorByName(String newColor)
    {
        this.color = newColor;
        this.redraw();
    }

    /**
     * Returns whether the rectangle is filled or not
     */
    public boolean isFilled()
    {
        return filled;
    }

    /**
     * Sets whether the rectangle should be filled; can either be true or false
     */
    public void setFilled(boolean newFilled)
    {
        this.filled = newFilled;
        this.redraw();
    }

    /**
     * Move the rectangle horizontally the specified number of pixels
     */
    public void moveHorizontal(int distance)
    {
        xPosition += distance;
        this.redraw();
    }

    /**
     * Move the rectangle vertically the specified number of pixels
     */
    public void moveVertical(int distance)
    {
        yPosition += distance;
        this.redraw();
    }

    /**
     * Removes the shape from the canvas
     * Wrapper around parent class
     */
    public void removeSelfFromCanvas()
    {
        super.removeSelfFromCanvas();
    }

    /**
     * Sends the shape to the "back" of the drawing (behind other shapes)
     */
    public void sendToBack()
    {
        super.sendToBack();
    }

    /**
     * Brings the shape to the "front" of the drawing (in front of other shapes)
     */
    public void bringToFront()
    {
        super.bringToFront();
    }

    
    /* (non-Javadoc)
     * @see DrawableShape#draw(java.awt.Graphics2D)
     */
    public void draw(Graphics2D g2d)
    {
        g2d.setColor(getColorFromString(color));
        if(filled)
            g2d.fillRect(xPosition,yPosition,width,height);
        else
            g2d.drawRect(xPosition,yPosition,width,height);
    }
    
    protected boolean contains(int x, int y)
    {
    	return (x >= xPosition) && (x <= xPosition+width) && (y >= yPosition) && (y <= yPosition+height);
    }

}
