import java.awt.*;
import java.awt.geom.*;

/**
 * An equilateral triangle that can be manipulated and drawn.
 * @author Joel Ross
 * @author adapted from Michael Kolling and David J. Barnes
 * @version Sp 2015
 *
 */
public class Triangle extends DrawableShape
{
    //instance variables
    private int xPosition;
    private int yPosition;
    private int sideLength;
    private int rotation;
    private String color;
    private boolean filled;

    /**
     * Creates a new Triangle object with default location, radius, and color.
     */
    public Triangle()
    {
        //give instance variables default values
        this.xPosition = 40;
        this.yPosition = 50;
        this.sideLength = 30;
        this.rotation = 0;
        this.color = "green";
        this.filled = true;

        this.addSelfToCanvas();
    }

    /**
     * Returns the triangle's current x position
     */
    public int getXPosition()
    {
        return xPosition;
    }

    /**
     * Sets the triangle's x position to the given value
     */
    public void setXPosition(int newX)
    {
        this.xPosition = newX;
        this.redraw();
    }

    /**
     * Returns the triangle's current y position
     */
    public int getYPosition()
    {
        return yPosition;
    }

    /**
     * Sets the triangle's x position to the given value
     */
    public void setYPosition(int newY)
    {
        this.yPosition = newY;
        this.redraw();
    }

    /**
     * Returns the triangle's current side length
     */
    public int getSideLength()
    {
        return sideLength;
    }

    /**
     * Sets the triangles's side length to the given value
     */
    public void setSideLength(int newSideLength)
    {
        this.sideLength = newSideLength;
        this.redraw();
    }

    /**
     * Returns the triangle's current rotation (in degrees)
     */
    public double getRotation()
    {
        return rotation;
    }

    /**
     * Set the triangle's current rotation (in degrees). Rotation is CLOCKWISE
     */
    public void setRotation(int newRotation)
    {
        this.rotation = newRotation;
        this.redraw();
    }

    /**
     * Returns the triangle's current color (as a String). Can either be: 
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public String getColorName()
    {
        return color;
    }

    /**
     * Set the circle's color to the given value. Can either be:
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public void setColorByName(String newColor)
    {
        this.color = newColor;
        this.redraw();
    }

    /**
     * Returns whether the triangle is filled or not
     */
    public boolean isFilled()
    {
        return filled;
    }

    /**
     * Sets whether the triangle is filled; can either be true or false
     */
    public void setFilled(boolean newFilled)
    {
        this.filled = newFilled;
        this.redraw();
    }

    /**
     * Move the triangle horizontally by the specified number of pixels.
     */
    public void moveHorizontal(int distance)
    {
        xPosition += distance;
        this.redraw();
    }

    /**
     * Move the triangle horizontally by the specified number of pixels.
     */
    public void moveVertical(int distance)
    {
        yPosition += distance;
        this.redraw();
    }

    /**
     * Removes the shape from the canvas
     * Wrapper around parent class
     */
    public void removeSelfFromCanvas()
    {
        super.removeSelfFromCanvas();
    }

    /**
     * Sends the shape to the "back" of the drawing (behind other shapes)
     */
    public void sendToBack()
    {
        super.sendToBack();
    }

    /**
     * Brings the shape to the "front" of the drawing (in front of other shapes)
     */
    public void bringToFront()
    {
        super.bringToFront();
    }

    
    /* (non-Javadoc)
     * @see DrawableShape#draw(java.awt.Graphics2D)
     */
    public void draw(Graphics2D g2d)
    {
        g2d.setColor(getColorFromString(color));

        double height = Math.sqrt(3)/2.0*sideLength; //the height of the triangle
        int[] xPoints = {xPosition, xPosition+sideLength/2, xPosition+sideLength}; //get the coordinates of the corners of the triangle
        int[] yPoints = {(int)(yPosition+height), yPosition, (int)(yPosition+height)};

        double radians = rotation*Math.PI/180.0; //convert to radians
        g2d.rotate(radians, xPosition+sideLength/2, yPosition+sideLength/2); //rotate the context around the center (converting to radians)

        if(filled)
            g2d.fill(new Polygon(xPoints, yPoints, 3));
        else
            g2d.draw(new Polygon(xPoints, yPoints, 3));

        g2d.rotate(-radians, xPosition+sideLength/2, yPosition+sideLength/2); //unrotate the context (again around the center)

    }
    
    protected boolean contains(int x, int y)
    {
    	//currently does not account for rotation!
    	double height = Math.sqrt(3)/2.0*sideLength; //the height of the triangle
        int[] xPoints = {xPosition, xPosition+sideLength/2, xPosition+sideLength}; //get the coordinates of the corners of the triangle
        int[] yPoints = {(int)(yPosition+height), yPosition, (int)(yPosition+height)};
        Polygon poly = new Polygon(xPoints, yPoints, 3);
        
        AffineTransform at = new AffineTransform(); //rotate the triangle without drawing
        at.rotate(Math.toRadians(rotation),xPosition+sideLength/2, yPosition+height/2);
        
        return at.createTransformedShape(poly).contains(x, y);
    }
}
